#version 430 core

in float Height;

out vec4 FragColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
} vert;

uniform vec3 camera_world;
uniform float brightness;
uniform bool celShading;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
	bool enabled;
} dirL;

const int MAXPOINTLIGHTS = 10;
uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
	bool enabled;
} pointL[MAXPOINTLIGHTS];

uniform samplerCube skybox;

const int CEL_SHADING_LEVELS = 6;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF) {
	float d = length(l);
	l = normalize(l);

	float diffuse = max(0, dot(n, l));
	if (celShading) {
		float level = floor(diffuse * CEL_SHADING_LEVELS);
		diffuse = level / CEL_SHADING_LEVELS;
	}

	vec3 r = reflect(-l, n);
	float specular = pow(max(0, dot(r, v)), 8.0);
	if (celShading) {
		float level = floor(specular * CEL_SHADING_LEVELS);
		specular = level / CEL_SHADING_LEVELS;
	}

	return diffuseF * diffuseC * diffuse + specularF * specularC * specular; 
}

void main()
{
    float h = (Height + 2) / 4.0;
	float level =  (celShading ? min(0.2 + floor(h * CEL_SHADING_LEVELS) / CEL_SHADING_LEVELS, 0.9) : h);

	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
    vec3 sky = texture(skybox, reflect(-v, n)).rgb;
	
	vec3 color;
	if (level > 0.4 && h > 0.4)
		color = vec3(level, level, level);
	else {	
		color = vec3(0.0, level * 0.4 - 0.05, level * 0.4 + 0.1);
	}

	FragColor = vec4(color * 0.8, 1.0); // ambient
	
	// add directional light contribution
	if (dirL.enabled) {
		FragColor.rgb += phong(n, -dirL.direction, v, dirL.color * color, 0.4, dirL.color, 0.0);
	}

	for(int i = 0; i < MAXPOINTLIGHTS; i++){
		if (pointL[i].enabled) {
			FragColor.rgb += phong(n, pointL[i].position - vert.position_world, v, pointL[i].color * color, 0.7, pointL[i].color, 0.3);
		}
	}

	if (level > 0.4 && h > 0.4)
		FragColor.rgb = FragColor.rgb * 0.8 + vec3(sky.r * 0.3, sky.g * 0.3, sky.b * 0.3);
	else
		FragColor.rgb = FragColor.rgb * 0.8 + vec3(0.0, sky.g * 0.3, sky.b * 0.3);

	FragColor.rgb *= max(0.1, brightness);
}