#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/


in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;
uniform float brightness;
uniform bool celShading;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;

uniform int textureType; // 1 - texture, 2 - color
uniform sampler2D diffuseTexture;
uniform vec3 textureColor;

const int CEL_SHADING_LEVELS = 4;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
	bool enabled;
} dirL;

const int MAXPOINTLIGHTS = 10;
uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
	bool enabled;
} pointL[MAXPOINTLIGHTS];

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);

	float diffuse = max(0, dot(n, l));
	//if (celShading) {
	//	float level = floor(diffuse * CEL_SHADING_LEVELS);
	//	diffuse = level / CEL_SHADING_LEVELS;
	//}

	vec3 r = reflect(-l, n);
	float specular = pow(max(0, dot(r, v)), alpha);
	//if (celShading) {
	//	specular = level / CEL_SHADING_LEVELS;
	//}

	float att = 1.0;	
	if (attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);

	return (diffuseF * diffuseC * diffuse + specularF * specularC * specular) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor;
	if (textureType == 1) {
		texColor = texture(diffuseTexture, vert.uv).rgb;
	} else if (textureType == 2) {
		texColor = textureColor;
	} else {
		texColor = vec3(0.0);
	}
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	// add directional light contribution
	if (dirL.enabled) {
		color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
	}
	
	// add point light contribution
    for(int i = 0; i < MAXPOINTLIGHTS; i++){
		if (pointL[i].enabled) {
			color.rgb += phong(n, pointL[i].position - vert.position_world, v, pointL[i].color * texColor, materialCoefficients.y, 
			                   pointL[i].color, materialCoefficients.z, specularAlpha, true, pointL[i].attenuation);
		}
	}
	
	color.rgb *= max(0.1, brightness);
}

